@@ -6,6 +6,7 @@ module.exports = exports = shrinkwrap
 var npm = require("./npm.js")
   , log = require("npmlog")
   , fs = require("fs")
+  , writeFileAtomic = require("write-file-atomic")
   , path = require("path")
   , readJson = require("read-package-json")
   , sortedObject = require("sorted-object")
@@ -59,9 +60,10 @@ function shrinkwrap_ (pkginfo, silent, dev, cb) {
 function save (pkginfo, silent, cb) {
   // copy the keys over in a well defined order
   // because javascript objects serialize arbitrarily
-  pkginfo.dependencies = sortedObject(pkginfo.dependencies)
+  pkginfo.dependencies = sortedObject(pkginfo.dependencies || {})
+  var swdata
   try {
-    var swdata = JSON.stringify(pkginfo, null, 2) + "\n"
+    swdata = JSON.stringify(pkginfo, null, 2) + "\n"
   } catch (er) {
     log.error("shrinkwrap", "Error converting package info to json")
     return cb(er)
@@ -69,7 +71,7 @@ function save (pkginfo, silent, cb) {
 
   var file = path.resolve(npm.prefix, "npm-shrinkwrap.json")
 
-  fs.writeFile(file, swdata, function (er) {
+  writeFileAtomic(file, swdata, function (er) {
     if (er) return cb(er)
     if (silent) return cb(null, pkginfo)
     console.log("wrote npm-shrinkwrap.json")
