@@ -1,19 +1,22 @@
 
 module.exports = star
 
-var url = require("url")
-  , npm = require("./npm.js")
+var npm = require("./npm.js")
   , registry = npm.registry
   , log = require("npmlog")
   , asyncMap = require("slide").asyncMap
+  , mapToRegistry = require("./utils/map-to-registry.js")
 
 star.usage = "npm star <package> [pkg, pkg, ...]\n"
            + "npm unstar <package> [pkg, pkg, ...]"
 
 star.completion = function (opts, cb) {
-  var uri = url.resolve(npm.config.get("registry"), "/-/short")
-  registry.get(uri, { timeout : 60000 }, function (er, list) {
-    return cb(null, list || [])
+  mapToRegistry("-/short", npm.config, function (er, uri) {
+    if (er) return cb(er)
+
+    registry.get(uri, { timeout : 60000 }, function (er, list) {
+      return cb(null, list || [])
+    })
   })
 }
 
@@ -24,13 +27,16 @@ function star (args, cb) {
     , using = !(npm.command.match(/^un/))
   if (!using) s = u
   asyncMap(args, function (pkg, cb) {
-    var uri = url.resolve(npm.config.get("registry"), pkg)
-    registry.star(uri, using, function (er, data, raw, req) {
-      if (!er) {
-        console.log(s + " "+pkg)
-        log.verbose("star", data)
-      }
-      cb(er, data, raw, req)
+    mapToRegistry(pkg, npm.config, function (er, uri) {
+      if (er) return cb(er)
+
+      registry.star(uri, using, function (er, data, raw, req) {
+        if (!er) {
+          console.log(s + " "+pkg)
+          log.verbose("star", data)
+        }
+        cb(er, data, raw, req)
+      })
     })
   }, cb)
 }
