@@ -46,7 +46,7 @@ function setup (cb) {
   cleanup()
   process.chdir(pkg)
 
-  var opts = { cache: path.resolve(pkg, "cache"), registry: common.registry};
+  var opts = { cache: path.resolve(pkg, "cache"), registry: common.registry}
   npm.load(opts, cb)
 }
 
